#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>

#include <util/delay.h>

#include "i2cmaster.h"

// use usbtiny library 
#include "usb.h"
#include "usbtiny.h"
typedef byte_t uchar;

#define USBDDR DDRD
#define USB_CFG_IOPORT PORTD

#define USB_CFG_DMINUS_BIT USBTINY_DMINUS
#define USB_CFG_DPLUS_BIT USBTINY_DPLUS

#define USBMASK (_BV(USBTINY_DPLUS) | _BV(USBTINY_DMINUS))

#define usbInit()  usb_init()
#define usbPoll()  usb_poll()


#define LED_DDR  DDRB						// Hardwarekonfiguration
#define LED_PORT PORTB
#define LED_PIN  PB1

#define ADDR_RTC 	0xA0

volatile unsigned char h_byte;	// Hight-Byte Temperatur-Registers LM75
volatile unsigned char l_byte;	// Low-Byte Temperatur-Registers LM75
volatile int click = 0;
volatile int lcd_intern = 1;


/* -------------------------------------------------------------------------- */
ISR(INT1_vect)
{
	LED_PORT ^= (1 << LED_PIN); 			// LED-PIN invertieren
}

/* -------------------------------------------------------------------------- */
void long_delay(uint16_t ms) {				// lange Pause
    for (; ms>0; ms--) _delay_ms(1);
}


/* -------------------------------------------------------------------------- */
unsigned char rtc_init(void)
{
	if (i2c_start(ADDR_RTC+I2C_WRITE)) return 1;
	i2c_write(0);					// Adresse 0
	i2c_write(0);					// Wert 0
	i2c_stop();
	long_delay(100);				// etwas warten bevor es weiter geht...
	return 0;
}

/* -------------------------------------------------------------------------- */
unsigned char rtc_read(unsigned char adr)
{
	unsigned char val; 

	if (i2c_start(ADDR_RTC+I2C_WRITE)) return 0;
	i2c_write(adr);					// zu lesende Adresse
	i2c_stop();
	long_delay(10);
	i2c_start(ADDR_RTC+I2C_READ);
	val = i2c_readNak();
	i2c_stop();
	return val;
}

/* -------------------------------------------------------------------------- */
unsigned char rtc_write(unsigned char adr, unsigned char val)
{
	if (i2c_start(ADDR_RTC+I2C_WRITE)) return 1;
	i2c_write(adr);					// Adresse
	i2c_write(val);					// Wert
	i2c_stop();
	return 0;
}


/* -------------------------------------------------------------------------- */
extern	byte_t	usb_setup ( byte_t data[8] )
{
	byte_t *replyBuf = data;
	uchar device = (data[1] >> 5);
	uchar command = (data[1] >> 2) & 3; // command 0 .. 7

	/* command format:
	 * 7 6 5 4 3 2 1 0
	 * D D D C C C L L
	 *
	 * DDD = Device
	 * CCC = Command
	 * LL  = number of bytes in transfer - 1
	 */

	switch (device) {

	case 0: // blit_board
		switch (command) {

		case 0: // echo
			replyBuf[0] = data[2];
			replyBuf[1] = data[3];
			return 2;
			break;

		default:
			break; 

		}
		break;


	case 3: // RTC_Board
    	switch(command) {
		case 0: // rtc_status
			replyBuf[0] = rtc_init();
			replyBuf[1] = 0;
			return 2;
			break;

	    case 1: // rtc_read
			replyBuf[0] = rtc_read(data[2]);
			return 1;
			break;

	    case 2: // rtc_write
			replyBuf[0] = rtc_write(data[2], data[3]);
			return 1;
			break;

		default:
			break; 
		}
		break;


	default:
	    break;
	}

  return 0;  // reply len
}


/* ------------------------------------------------------------------------- */
/* ------------------------------------------------------------------------- */
int	main(void) {

	LED_DDR   = (1 << LED_PIN);		// LED-Pin als Ausgang schalten
	LED_PORT &= ~(1 << LED_PIN);

	MCUCR |= (1<<ISC10) | (1<<ISC11);	// steigende Flanke (L->H) fuer INT1
	GICR  |= (1<<INT1);          		// externer Interrupt an INT1

	usbInit();				// USB initialisieren
	i2c_init();				// TWI initialisieren
	rtc_init();				// RTC initialisieren

	sei();

	for(;;) {	
		usbPoll();
	}
	return 0;
}

/* ------------------------------------------------------------------------- */
